// C Source File
// Created 9/3/2002; 5:29:21 PM

/*****************************************************
*
*                Compiler Directives
*
*****************************************************/

#define USE_TI89              // Produce .89z File

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define SAVE_SCREEN           // Save/Restore LCD Contents


/*****************************************************
*
*                     Includes
*
*****************************************************/

//#include <tigcclib.h>         // Include All Header Files
#include <intr.h>
#include <gray.h>
#include <alloc.h>
#include <kbd.h>
#include <string.h>
#include <peekpoke.h>
#include <system.h>
#include <stdlib.h>
#include "Snake.h"

/*// Debug includes
#include "extgraph.h"
#include <graph.h>
*/

/*****************************************************
*
*                 Global variables
*
*****************************************************/

char _comment[] = "Snake v2.0";
INT_HANDLER oldInt1 = NULL, oldInt5 = NULL; // Interrupt handlers
volatile int count = 0; //Used to time

/************************************************
*
*  TimerHandler - Decrements variable count every call
*   !Interrupt Handler!
*
************************************************/
DEFINE_INT_HANDLER (TimerHandler) {
	if(count)
		count--;
	//ExecuteHandler(oldInt5);
}

/*****************************************************
*
*                   Functions
*
*****************************************************/

/************************************************
*
*  _main() - Initializes game, displays intros,
*             and then launches game
*   !Used as program entry point only!
*
************************************************/

void _main(void) {
	//Setup
	
	//Interrupt handler 1 interferes with grayscale. This shuts it off
	//Uninstall old AutoInt 1 by replacing it with a dummy interrupt handler
	oldInt1 = GetIntVec(AUTO_INT_1);
	SetIntVec(AUTO_INT_1, DUMMY_HANDLER);
	
	//seed the random number generator
	randomize();
	
	//change the constrast to make the grayscale look nicer
	OSContrastDn();
	OSContrastDn();
	//Turn on grayscale
	GrayOn();
	
	//Install timer
	oldInt5 = GetIntVec(AUTO_INT_5);
	SetIntVec(AUTO_INT_5, TimerHandler);
	//Set up the programmable rate generator
	//Send a value (0xFD) to port 0x600017. The rate generator counts up from this value,
	//causing an interrupt (AUTO_INT_5) when overflow occurs (8-bit value). Then the rate generator resets to this value and counts again.
	pokeIO(0x600017, 0xFD);
	
	//wipeScreen();
	FirewreathLogo();
	
	if(needScoreList()) createScoreList();
	
	doIntro();
	
	OSContrastUp();
	menu();
	
	//Clean up
	//Turn off grayscale if it's still on
	GrayOff();
	OSContrastUp();
	
	//Uninstall timer
	SetIntVec(AUTO_INT_5, oldInt1);
	pokeIO(0x600017, 0xB2);
	
	//Reinstall old AutoInt1
	SetIntVec(AUTO_INT_1, oldInt1);
}


/************************************************
*
*  delay() - Waits for a desired amount of time
*
*  int delayLen - How long to wait
*
************************************************/
void delay(int delayLen) {
	count = delayLen;
	while(count);
}


/************************************************
*
*  optDelay() - Waits for a desired amount of time,
*                or until a keypress (optional delay)
*
*  int delayLen - How long to wait
*
************************************************/
void optDelay(int delayLen) {
	count = delayLen;
	while(count && !_rowread(0));
}


/************************************************
*
*  contOptDelay() - Waits for a desired amount of time,
*                    or until a keypress (optional delay)
*
*  int delayLen - How long to wait, 0 to continue last delay
*
*  return int  -  How many intervals left upon returning
*
************************************************/

int contOptDelay(int delayLen) {
	if(delayLen)
		count = delayLen;
	while(count && !_rowread(0));
	return count;
}


/************************************************
*
*  getKey() - Searches for keypresses
*
*  return int - Which keys were pressed
*
************************************************/

int getKey(void) {
	int key = KEY_NONE;
	
	int temp = _rowread(ROW_ARROW);
	if(temp & COL_LEFT) key |= KEY_LEFT;
	if(temp & COL_RIGHT) key |= KEY_RIGHT;
	if(temp & COL_DOWN) key |= KEY_DOWN;
	if(temp & COL_UP) key |= KEY_UP;
	//temp = _rowread(ROW_MODIFIER);
	if(temp & COL_2ND) key |= KEY_2ND;
	if(temp & COL_DMND)	key |= KEY_DMND;
	temp = _rowread(ROW_ENTER);
	if(temp & COL_ENTER) key |= KEY_ENTER;
	if(temp & COL_FUNC) key |= KEY_F5;
	if(_rowread(ROW_ESC) & COL_ESC) key |= KEY_ESC;
	if(_rowread(ROW_F1) & COL_FUNC) key |= KEY_F1;
	//if(_rowread(ROW_BOSS) & COL_BOSS) key |= KEY_BOSS;
	
	return key;
}


/************************************************
*
*  getAlpha() - Searches for keypresses, checking
*                for characters in addition
*
*  return int - The keypress found, encoded as
*                ngetchx() would
*
************************************************/

int getAlpha(void) {
	int key = KEY_NONE;
	int row;
	
	row = _rowread(ROW_ARROW);
	if(row & COL_LEFT) key = OSKEY_LEFT;
	if(row & COL_DOWN) key = OSKEY_DOWN;
	if(row & COL_UP) key = OSKEY_UP;
	if(row & COL_RIGHT) key = OSKEY_RIGHT;
	
	row = _rowread(ROW_EU);
	if(row & COL_E) key = 'E';//key |= KEY_E;
	if(row & COL_J) key = 'J';//key |= KEY_J;
	if(row & COL_O) key = 'O';//key |= KEY_O;
	if(row & COL_U) key = 'U';//key |= KEY_U;
	if(row & COL_NTR) key = OSKEY_ENTER;//key |= KEY_NTR;
	
	row = _rowread(ROW_T_);
	if(row & COL_BCK) key = OSKEY_BACKSPACE;//key |= KEY_BCK;
	if(row & COL_T) key = 'T';//key |= KEY_T;
	if(row & COL_D) key = 'D';//key |= KEY_D;
	if(row & COL_I) key = 'I';//key |= KEY_I;
	if(row & COL_N) key = 'N';//key |= KEY_N;
	if(row & COL_S) key = 'S';//key |= KEY_S;
	if(row & COL__) key = ' ';//key |= KEY__;
	
	row = _rowread(ROW_ZW);
	if(row & COL_Z) key = 'Z';//key |= KEY_Z;
	if(row & COL_C) key = 'C';//key |= KEY_C;
	if(row & COL_H) key = 'H';//key |= KEY_H;
	if(row & COL_M) key = 'M';//key |= KEY_M;
	if(row & COL_R) key = 'R';//key |= KEY_R;
	if(row & COL_W) key = 'W';//key |= KEY_W;
	
	row = _rowread(ROW_YV);
	if(row & COL_Y) key = 'Y';//key |= KEY_Y;
	if(row & COL_B) key = 'B';//key |= KEY_B;
	if(row & COL_G) key = 'G';//key |= KEY_G;
	if(row & COL_L) key = 'L';//key |= KEY_L;
	if(row & COL_Q) key = 'Q';//key |= KEY_Q;
	if(row & COL_V) key = 'V';//key |= KEY_V;
	
	row = _rowread(ROW_XP);
	if(row & COL_X) key = 'X';//key |= KEY_X;
	if(row & COL_A) key = 'A';//key |= KEY_A;
	if(row & COL_F) key = 'F';//key |= KEY_F;
	if(row & COL_K) key = 'K';//key |= KEY_K;
	if(row & COL_P) key = 'P';//key |= KEY_P;
	
	if(_rowread(ROW_ESC) & COL_ESC) key = OSKEY_ESC;
	
	if(!key & _rowread(0)) key = 1;//key |= KEY_OTHER;
	
	return key;
}


/************************************************
*
*  initSnake() - Initializes a snake structure
*
*  SNAKE *snake - A pointer to the snake structure
*
************************************************/

void initSnake(SNAKE *snake) {
	snake->head.x = 75;
	snake->head.y = 85;
	snake->tail.x = 75;
	snake->tail.y = 85;
	snake->headDir = DIR_NORTH;
	snake->tailDir = DIR_NORTH;
	snake->lives = 3;
	snake->len = 1;
	snake->grow = 5;
	snake->dead = STATUS_ALIVE;
	snake->eating = 0;
	snake->turns = 0;
	snake->turn = (TURN *)malloc(50 * sizeof(TURN));
	if(snake->turn)
		memset(snake->turn, 0, 50 * sizeof(TURN));
}


/************************************************
*
*  initBoard() - Initializes a board
*
*  int level  -  Which level to use (> 0)
*
*  return char* - Points to the board if successful
*                  NULL if unsuccessful
*
************************************************/

char* initBoard(int level) {
	unsigned int numAlloc = 0, width = DEFAULT_WIDTH, height = DEFAULT_HEIGHT;
	int i = 0;
	char *board = NULL;
	
	if(level > 9) {
		i = level - 7;
		return makeRandomBoard(&i);
	}
	
	switch(level) {
	case LEVEL_5:
		//height--;
		break;
	case LEVEL_6:
		//height--;
		width -= 3;
		break;
	case LEVEL_7:
		//height--;
		width -= 4;
		break;
	case LEVEL_8:
		//height--;
		width -= 9;
		break;
	case LEVEL_9:
		width -= 3;
		break;
	/*case CUSTOM_LEVEL:
		return loadBoard(NULL);
		break;
	case RANDOM_LEVEL:
		i = -1;
		return makeRandomBoard(&i);
		break;*/
	}
	
	numAlloc = (width*height+BOARD_TO_DATA) * sizeof(char);
	
	board = (char *)malloc(numAlloc);
	
	if(!board)
		return NULL;
	
	memset(board, 0, numAlloc);
	*(board + BOARD_TO_MAX_X) = width;
	*(board + BOARD_TO_MAX_Y) = height;
	BOARDWRAP = 0;
	
	switch(level) {
	case LEVEL_0:
		BOARDMINAPPLES = 2;
		BOARDMAXAPPLES = 8;
		BOARDWRAP = 1;
		break;
	case LEVEL_1:
		for(i = 5; i < 25; i++)
			BOARDPOS(i, 7) = BOARD_OBSTACLE;    // ---- 
		BOARDMINAPPLES = 2;
		BOARDMAXAPPLES = 10;
		BOARDWRAP = 1;
		break;
	case LEVEL_2:
		for(i = 2; i < 13; i++) {
			BOARDPOS(5, i) = BOARD_OBSTACLE;
			BOARDPOS(24, i) = BOARD_OBSTACLE;   // | |
		}
		BOARDMINAPPLES = 1;
		BOARDMAXAPPLES = 12;
		BOARDWRAP = 1;
		break;
	case LEVEL_3:
		for(i = 3; i < 27; i++)
			BOARDPOS(i, 7) = BOARD_OBSTACLE;    // ++
		for(i = 2; i < 13; i++) {
			//BOARDPOS(14, i) = BOARD_OBSTACLE;
			BOARDPOS(9, i) = BOARD_OBSTACLE;
			BOARDPOS(20, i) = BOARD_OBSTACLE;
		}
		BOARDMINAPPLES = 1;
		BOARDMAXAPPLES = 14;
		BOARDWRAP = 1;
		break;
	case LEVEL_4:
		for(i = 3; i < 14; i++) {
			BOARDPOS(i, 2) = BOARD_OBSTACLE;
			BOARDPOS(29 - i, 2) = BOARD_OBSTACLE;  //[ ]
			BOARDPOS(i, 12) = BOARD_OBSTACLE;
			BOARDPOS(29 - i, 12) = BOARD_OBSTACLE;
		}
		for(i = 3; i < 6; i++) {
			BOARDPOS(3, i) = BOARD_OBSTACLE;
			BOARDPOS(3, 14-i) = BOARD_OBSTACLE;
			BOARDPOS(26, i) = BOARD_OBSTACLE;
			BOARDPOS(26, 14-i) = BOARD_OBSTACLE;
		}
		BOARDMINAPPLES = 1;
		BOARDMAXAPPLES = 16;
		break;
	case LEVEL_5:
		for(i = 0; i < 14; i++) {
			BOARDPOS(i, 7) = BOARD_OBSTACLE;
			BOARDPOS(29 - i, 7) = BOARD_OBSTACLE;
		}
		for(i = 0; i < 5; i++) {
			BOARDPOS(8, i) = BOARD_OBSTACLE;
			BOARDPOS(21, i) = BOARD_OBSTACLE;
			BOARDPOS(8, 14-i) = BOARD_OBSTACLE;
			BOARDPOS(21, 14-i) = BOARD_OBSTACLE;
		}
		BOARDMINAPPLES = 1;
		BOARDMAXAPPLES = 16;
		break;
	case LEVEL_6:
		for(i = 0; i < 5; i++) {
			BOARDPOS(10, i) = BOARD_OBSTACLE;
			BOARDPOS(16, i) = BOARD_OBSTACLE;
			BOARDPOS(10, 14-i) = BOARD_OBSTACLE;
			BOARDPOS(16, 14-i) = BOARD_OBSTACLE;
		}
		for(i = 4; i < 11; i++) {
			BOARDPOS(i, 5) = BOARD_OBSTACLE;
			BOARDPOS(26-i, 5) = BOARD_OBSTACLE;
			BOARDPOS(i, 9) = BOARD_OBSTACLE;
			BOARDPOS(26-i, 9) = BOARD_OBSTACLE;
		}
		BOARDMINAPPLES = 1;
		BOARDMAXAPPLES = 16;
		break;
	case LEVEL_7:
		for(i = 2; i < 10; i++) {
			BOARDPOS(i, 2) = BOARD_OBSTACLE;
			BOARDPOS(25-i, 2) = BOARD_OBSTACLE;
			BOARDPOS(i, 12) = BOARD_OBSTACLE;
			BOARDPOS(25-i, 12) = BOARD_OBSTACLE;
		}
		for(i = 3; i < 6; i++) {
			BOARDPOS(2, i) = BOARD_OBSTACLE;
			BOARDPOS(23, i) = BOARD_OBSTACLE;
			BOARDPOS(2, 14-i) = BOARD_OBSTACLE;
			BOARDPOS(23, 14-i) = BOARD_OBSTACLE;
			BOARDPOS(6, i+3) = BOARD_OBSTACLE;
			BOARDPOS(19, i+3) = BOARD_OBSTACLE;
			BOARDPOS(i+7, 7) = BOARD_OBSTACLE;
			BOARDPOS(i+10, 7) = BOARD_OBSTACLE;
		}
		for(i = 11; i < 15; i++) {
			BOARDPOS(i, 3) = BOARD_OBSTACLE;
			BOARDPOS(i, 11) = BOARD_OBSTACLE;
		}
		BOARDMINAPPLES = 1;
		BOARDMAXAPPLES = 16;
		break;
	case LEVEL_8:
		for(i = 0; i < 4; i++) {
			BOARDPOS(i, 9) = BOARD_OBSTACLE;
			BOARDPOS(i+4, 9) = BOARD_OBSTACLE;
			BOARDPOS(i+13, 5) = BOARD_OBSTACLE;
			BOARDPOS(i+17, 5) = BOARD_OBSTACLE;
			BOARDPOS(12, i+2) = BOARD_OBSTACLE;
			BOARDPOS(8, i+9) = BOARD_OBSTACLE;
		}
		for(i = 0; i < 5; i++) {
			BOARDPOS(8, i) = BOARD_OBSTACLE;
			BOARDPOS(i+4, 5) = BOARD_OBSTACLE;
			BOARDPOS(12, i+10) = BOARD_OBSTACLE;
			BOARDPOS(i+12, 9) = BOARD_OBSTACLE;
		}
		for(i = 0; i < 2; i++) {
			BOARDPOS(i+6, 6) = BOARD_OBSTACLE;
			BOARDPOS(i+13, 6) = BOARD_OBSTACLE;
			BOARDPOS(i+6, 8) = BOARD_OBSTACLE;
			BOARDPOS(i+13, 8) = BOARD_OBSTACLE;
			BOARDPOS(i+8, 4) = BOARD_OBSTACLE;
			BOARDPOS(i+11, 4) = BOARD_OBSTACLE;
			BOARDPOS(i+8, 10) = BOARD_OBSTACLE;
			BOARDPOS(i+11, 10) = BOARD_OBSTACLE;
		}
		BOARDMINAPPLES = 1;
		BOARDMAXAPPLES = 16;
		break;
	case LEVEL_9:
		for(i = 0; i < 6; i++) {
			BOARDPOS(3, i) = BOARD_OBSTACLE;
			BOARDPOS(3, 14-i) = BOARD_OBSTACLE;
			BOARDPOS(9, i) = BOARD_OBSTACLE;
			BOARDPOS(9, 14-i) = BOARD_OBSTACLE;
			BOARDPOS(17, i) = BOARD_OBSTACLE;
			BOARDPOS(17, 14-i) = BOARD_OBSTACLE;
			BOARDPOS(23, i) = BOARD_OBSTACLE;
			BOARDPOS(23, 14-i) = BOARD_OBSTACLE;
		}
		for(i = 2; i < 13; i++) {
			BOARDPOS(6, i) = BOARD_OBSTACLE;
			BOARDPOS(12, i) = BOARD_OBSTACLE;
			BOARDPOS(14, i) = BOARD_OBSTACLE;
			BOARDPOS(20, i) = BOARD_OBSTACLE;
		}
		for(i = 0; i < 2; i++) {
			BOARDPOS(i, 7) = BOARD_OBSTACLE;
			//BOARDPOS(i, 8) = BOARD_OBSTACLE;
			BOARDPOS(26-i, 7) = BOARD_OBSTACLE;
			//BOARDPOS(26-i, 8) = BOARD_OBSTACLE;
			BOARDPOS(i+11, 7) = BOARD_OBSTACLE;
			//BOARDPOS(i+11, 8) = BOARD_OBSTACLE;
			BOARDPOS(i+14, 7) = BOARD_OBSTACLE;
			//BOARDPOS(i+14, 8) = BOARD_OBSTACLE;
		}
		for(i = 5; i < 8; i++) {
			BOARDPOS(i, 7) = BOARD_OBSTACLE;
			//BOARDPOS(i, 8) = BOARD_OBSTACLE;
			BOARDPOS(i+14, 7) = BOARD_OBSTACLE;
			//BOARDPOS(i+14, 8) = BOARD_OBSTACLE;
		}
		BOARDMINAPPLES = 1;
		BOARDMAXAPPLES = 16;
		break;
	case LEVEL_FREEPLAY:
		BOARDMINAPPLES = 1;
		BOARDMAXAPPLES = -1;
		BOARDWRAP = 1;
		break;
	}
	
	return board;
}


/************************************************
*
*  game() - Plays the game
*
*  int speed  -   The speed of the snake
*  int type   -   The type of game:
*                   GAME_FREEPLAY
*                   GAME_WARP
*                   GAME_CUSTOM
*                   GAME_RANDOM
*                   GAME_LEVELS
*
*  return SCORE - The ending score
*
************************************************/

SCORE game(int speed, int type) {
	SNAKE snake;
	int speeds[9] = {80, 72, 64, 55, 45, 35, 24, 14, 10};
	char *board = NULL;
	char levelName[9] = "Freeplay";
	int key = KEY_NONE, apples = 0, level = LEVEL_0, eaten = 0;
	unsigned long score = 0, lastScore = 0;
	SCORE endScore = {"NONE", 0};
	int rndDiff = -1, nextDiff;
	
	initSnake(&snake);
	if(!snake.turn)
		return endScore;
	
	if(type == GAME_FREEPLAY) {
		level = LEVEL_FREEPLAY;
		snake.lives = 1;
	}
	if(type == GAME_WARP) {
		if((level = askWarp()) == NO_LEVEL)
			return endScore;
	}
	if(type == GAME_CUSTOM) {
		level = CUSTOM_LEVEL;
		board = loadBoard(levelName);
		snake.lives = 1;
	}
	if(type == GAME_RANDOM) {
		level = RANDOM_LEVEL;
		//board = newRandomBoard(RNDM_PROMPT);
		board = makeRandomBoard(&rndDiff);
		nextDiff = rndDiff;
		snake.lives = 1;
		sprintf(levelName, "Random");
		//rndDiff = getRandomDifficulty();
	}
	
	while(snake.lives) {
		if(snake.dead == STATUS_NEXT_LEVEL)
			showPass(++level);
		if(type >= 0)
			board = initBoard(level);
		if(!board) {
			free(snake.turn);
			return endScore;
		}
		eaten = 0;
		apples = 0;
		snake.head.x = MIDBOARD;
		snake.head.y = BOTTOMBOARD;
		snake.tail.x = MIDBOARD;
		snake.tail.y = BOTTOMBOARD;
		snake.headDir = DIR_NORTH;
		snake.tailDir = DIR_NORTH;
		snake.len = 1;
		snake.grow = 5;
		snake.turns = 0;
		memset(snake.turn, 0, 50 * sizeof(TURN));
		snake.dead = STATUS_ALIVE;
		lastScore = score;
		
		if(type == GAME_LEVELS || type == GAME_WARP)
			doLevelIntro(level);
		else
			doCustomIntro(levelName);
		drawBoard(board, 1);
		drawHUD(&snake);
		if(type == GAME_FREEPLAY)
			updateHUD(eaten, score);
		else
			updateHUD((int)BOARDMAXAPPLES - eaten, score);
		countdown();
		
		while(!(snake.dead)) {
			if(apples < BOARDMINAPPLES) {
				createApple(board);
				apples++;
			}
			delay(speeds[speed]);
			key = getKey();
			if(key & KEY_BOSS) {
				free(snake.turn);
				free(board);
				forceExit();
			}
			if(key & KEY_DOWN && snake.headDir & 1) {
				snake.headDir = DIR_SOUTH;
				snake.turn[snake.turns].pos.x = snake.head.x;
				snake.turn[snake.turns].pos.y = snake.head.y;
				snake.turn[snake.turns].dir = snake.headDir;
				snake.turns++;
			}
			else if(key & KEY_LEFT && !(snake.headDir & 1)) {
				snake.headDir = DIR_WEST;
				snake.turn[snake.turns].pos.x = snake.head.x;
				snake.turn[snake.turns].pos.y = snake.head.y;
				snake.turn[snake.turns].dir = snake.headDir;
				snake.turns++;
			}
			else if(key & KEY_UP && snake.headDir & 1) {
				snake.headDir = DIR_NORTH;
				snake.turn[snake.turns].pos.x = snake.head.x;
				snake.turn[snake.turns].pos.y = snake.head.y;
				snake.turn[snake.turns].dir = snake.headDir;
				snake.turns++;
			}
			else if(key & KEY_RIGHT && !(snake.headDir & 1)) {
				snake.headDir = DIR_EAST;
				snake.turn[snake.turns].pos.x = snake.head.x;
				snake.turn[snake.turns].pos.y = snake.head.y;
				snake.turn[snake.turns].dir = snake.headDir;
				snake.turns++;
			}
			if(key & KEY_ESC)
				snake.dead = DEATH_ESC;
			updateSnake(&snake, board);
			if(snake.eating > 0) {
				snake.eating--;
				eaten++;
				snake.grow += 2;
				score += speed + 1;
				apples--;
				if(type == GAME_FREEPLAY)
					updateHUD(eaten, score);
				else
					updateHUD((int)BOARDMAXAPPLES - eaten, score);
				if(eaten == BOARDMAXAPPLES)
					snake.dead = STATUS_NEXT_LEVEL;
			}
		}
		
		free(board);
		if(snake.dead == DEATH_ESC) {
			snake.lives = 0;
		}
		else if(snake.dead == STATUS_NEXT_LEVEL) {
			winSequence(&snake, board, speeds[speed]/3);
		}
		else {
			drawDeath(snake.dead);
			if(snake.lives) score = lastScore;
		}
		if(type == GAME_CUSTOM && snake.dead == STATUS_NEXT_LEVEL)
			board = loadBoardFile(levelName);
			//snake.lives = 0;
		if(type == GAME_RANDOM && snake.dead == STATUS_NEXT_LEVEL) {
			//board = newRandomBoard(RNDM_ADVANCE);
			nextDiff++;
			board = makeRandomBoard(&nextDiff);
			sprintf(levelName, "Harder");
		}
	}
	
	free(snake.turn);
	
	if(type == GAME_LEVELS || type == GAME_WARP)
		sprintf(endScore.level, "%d", level);
	else if(type == GAME_RANDOM) {
		sprintf(endScore.level, "Rnd%d", rndDiff);
	}
	else if(type == GAME_CUSTOM) {
		endScore.level[0] = levelName[0];
		endScore.level[1] = levelName[1];
		endScore.level[2] = levelName[2];
		endScore.level[3] = levelName[3];
		endScore.level[4] = 0;
	}
	else
		sprintf(endScore.level, "L");
		//endScore.level = level;
	endScore.score = score;
	return endScore;
}


/************************************************
*
*  forceExit() - Uninitializes games, then quits
*   !Does not free allocated pointers!
*
************************************************/

void forceExit(void) {
	//Clean up
	//Turn off grayscale if it's still on
	GrayOff();
	
	OSContrastUp();
	//Uninstall timer
	SetIntVec(AUTO_INT_5, oldInt1);
	pokeIO(0x600017, 0xB2);
	
	//Reinstall old AutoInt1
	SetIntVec(AUTO_INT_1, oldInt1);
	exit(0);
}